//
//  K8055.m
//  K8055Demo
//
//  Created by Peter Henderson on 11/09/05.
//  Copyright 2005 Control Plus. All rights reserved.
//

#import "K8055.h"
NSString *PRHNewDataFromK8055 = @"PRHNewDataFromK8055";


@implementation K8055
- (id)init
{
    [super init];
    if (self)
    {
		cardAddress = 0;
		online = 0;
		DA1 = 0;
		DA2 = 0;
		AD1 = 0;
		AD2 = 0;
		inputValue = 0;
		outputValue = 0;
		counter1 = 0;
		counter2 = 0;
		k8055Interface = NULL;
		outputTest = NO;
    }
    return self;
}

//--------------------------------------------------------------
- (void)dealloc
{
    [super dealloc];
}
//--------------------------------------------------------------
-(BOOL)open
{
// open the hardware
    CFMutableDictionaryRef 	matchingDict;
    CFNumberRef			numberRef;
    kern_return_t		kr;
	io_iterator_t myIterator;
	io_object_t nextObject;
    IOCFPlugInInterface **plugInInterface = NULL;
    SInt32 score;
	IOHIDDeviceInterface **deviceInterface = NULL;
	long usbProduct = 0x5500 + cardAddress;
	long usbVendor = kVellemanVendorID;
	

    matchingDict = IOServiceMatching(kIOHIDDeviceKey);
    if (!matchingDict)
    {
        printf("Can't create a HID matching dictionary\n");
        return NO;
    }

    numberRef = CFNumberCreate(kCFAllocatorDefault, kCFNumberSInt32Type, &usbVendor);
    CFDictionarySetValue(matchingDict, CFSTR(kIOHIDVendorIDKey), numberRef);
    CFRelease(numberRef);
	
    numberRef = CFNumberCreate(kCFAllocatorDefault, kCFNumberSInt32Type, &usbProduct);
    CFDictionarySetValue(matchingDict, CFSTR(kIOHIDProductIDKey), numberRef);
    CFRelease(numberRef);
    	
	kr = IOServiceGetMatchingServices(kIOMasterPortDefault, matchingDict,&myIterator);
	if (kr)
    {
        printf("ERR: IOServiceGetMatchingServices (%08x)\n", kr);
		return NO;
    }
	
	if(nextObject = IOIteratorNext(myIterator))
	{
		kr = IOCreatePlugInInterfaceForService(nextObject, kIOHIDDeviceUserClientTypeID, kIOCFPlugInInterfaceID,&plugInInterface, &score);

        if ((kIOReturnSuccess != kr) || !plugInInterface)
        {
            printf("ERR: IOCreatePlugInInterfaceForService (%08x)\n", kr);
			return NO;
        }
		
        // Use the plugin interface to retrieve the device interface.
        kr = (*plugInInterface)->QueryInterface(plugInInterface, CFUUIDGetUUIDBytes(kIOHIDDeviceInterfaceID),(LPVOID) &deviceInterface);
        if (kr || !deviceInterface)
        {
            printf("ERR: QueryInterface (%08x)\n", (int) kr);
			return NO;
        }
				
		kr = (*deviceInterface)->open(deviceInterface,0);
		if ((kr != kIOReturnSuccess))
		{
			printf("ERR: open (%08x)\n", kr);
			return NO;
		}
		k8055Interface = deviceInterface;
		
		[NSThread detachNewThreadSelector:@selector(idleThread:) toTarget:self withObject:nil];

	
	}
	return YES;



}
//--------------------------------------------------------------
-(void)close
{
// close the hardware


}
//--------------------------------------------------------------------------------------
-(void) postDataChangedNotification
{
    NSNotification *newDataNotification = [NSNotification notificationWithName:PRHNewDataFromK8055 object:self];
    [[NSNotificationQueue defaultQueue] enqueueNotification:newDataNotification postingStyle:NSPostWhenIdle coalesceMask:NSNotificationCoalescingOnSender forModes:nil];
}
//--------------------------------------------------------------------------------------
- (void)idleThread:(id)anObject
{
    float sleeptime = 1.0;
    float st = 100;
    int loopcount = 0;
    int i;
    BOOL changed;
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	kern_return_t		kr;
	IOHIDEventStruct valueEvent;
	IOHIDEventStruct valueEventB;


    sleeptime = 0.1;
    
    [NSThread sleepUntilDate:[NSDate dateWithTimeIntervalSinceNow:sleeptime]];

    while (1)
    {
		changed = NO;
        kr = (*k8055Interface)->getElementValue(k8055Interface, 2 , &valueEvent);	// get the digital inputs
		if ((kr != kIOReturnSuccess))
		{
			printf("ERR0: getElementValue (%08x)\n", kr);
		}
		if(lastDin!=valueEvent.value)
		{
		// digital inputs has changed
		// post a notification to all thos who are listening
			lastDin = valueEvent.value;
			if(valueEvent.value&0x10)
				inputValue = (inputValue & 0xFE) + 0x01;
			else
				inputValue = (inputValue & 0xFE);
			if(valueEvent.value&0x20)
				inputValue = (inputValue & 0xFD) + 0x02;
			else
				inputValue = (inputValue & 0xFD);
			if(valueEvent.value&0x01)
				inputValue = (inputValue & 0xFB) + 0x04;
			else
				inputValue = (inputValue & 0xFB);
			if(valueEvent.value&0x40)
				inputValue = (inputValue & 0xF7) + 0x08;
			else
				inputValue = (inputValue & 0xF7);
			if(valueEvent.value&0x80)
				inputValue = (inputValue & 0xEF) + 0x10;
			else
				inputValue = (inputValue & 0xEF);
			changed = YES;
		}
        kr = (*k8055Interface)->getElementValue(k8055Interface, 4 , &valueEvent);	// get the analog input 1
		if ((kr != kIOReturnSuccess))
		{
			printf("ERR1: getElementValue (%08x)\n", kr);
		}
		if(AD1!=valueEvent.value)
		{
			changed = YES;
			AD1=valueEvent.value;
		
		}
        kr = (*k8055Interface)->getElementValue(k8055Interface, 5 , &valueEvent);	// get the analog input 2
		if ((kr != kIOReturnSuccess))
		{
			printf("ERR2: getElementValue (%08x)\n", kr);
		}
		if(AD2!=valueEvent.value)
		{
			changed = YES;
			AD2=valueEvent.value;
		}
        kr = (*k8055Interface)->getElementValue(k8055Interface, 6 , &valueEvent);	// get counter 1 LSB
		if ((kr != kIOReturnSuccess))
		{
			printf("ERR3: getElementValue (%08x)\n", kr);
		}
        kr = (*k8055Interface)->getElementValue(k8055Interface, 7 , &valueEventB);	// get counter 1 MSB
		if ((kr != kIOReturnSuccess))
		{
			printf("ERR4: getElementValue (%08x)\n", kr);
		}
		int newcount = valueEvent.value + 256*valueEventB.value;
		if(counter1!=newcount)
		{
			changed = YES;
			counter1=newcount;
		}
        kr = (*k8055Interface)->getElementValue(k8055Interface, 8 , &valueEvent);	// get counter 2 LSB
		if ((kr != kIOReturnSuccess))
		{
			printf("ERR5: getElementValue (%08x)\n", kr);
		}
        kr = (*k8055Interface)->getElementValue(k8055Interface, 9 , &valueEventB);	// get counter 2 MSB
		if ((kr != kIOReturnSuccess))
		{
			printf("ERR6: getElementValue (%08x)\n", kr);
		}
		newcount = valueEvent.value + 256*valueEventB.value;
		if(counter2!=newcount)
		{
			changed = YES;
			counter2=newcount;
		}
		kr = (*k8055Interface)->getElementValue(k8055Interface, 10, &valueEvent);	
		valueEvent.value = 0x05;
		kr = (*k8055Interface)->setElementValue(k8055Interface, 10, &valueEvent,0,0,nil,nil);	

		kr = (*k8055Interface)->getElementValue(k8055Interface, 11, &valueEvent);
		if(outputTest)
		{
			outputValue++;
			if(outputValue==0)
				outputTest = NO;
		}
		valueEvent.value = outputValue;
		kr = (*k8055Interface)->setElementValue(k8055Interface, 11, &valueEvent,0,0,nil,nil);	
		if ((kr != kIOReturnSuccess))
		{
			printf("ERR0: setElementValue (%08x)\n", kr);
		}
		kr = (*k8055Interface)->getElementValue(k8055Interface, 12, &valueEvent);	
		valueEvent.value = DA1;
		kr = (*k8055Interface)->setElementValue(k8055Interface, 12, &valueEvent,0,0,nil,nil);	
		if ((kr != kIOReturnSuccess))
		{
			printf("ERR1: setElementValue (%08x)\n",i, kr);
		}
		kr = (*k8055Interface)->getElementValue(k8055Interface, 13, &valueEvent);	
		valueEvent.value = DA2;
		kr = (*k8055Interface)->setElementValue(k8055Interface, 13, &valueEvent,0,0,nil,nil);	
		if ((kr != kIOReturnSuccess))
		{
			printf("ERR2: setElementValue (%08x)\n", kr);
		}
		if(changed)
			[self performSelectorOnMainThread:@selector(postDataChangedNotification) withObject:nil waitUntilDone:NO];
		[NSThread sleepUntilDate:[NSDate dateWithTimeIntervalSinceNow:sleeptime]];
        if(loopcount++>100)
        {
            if(pool)
                [pool release];
            pool = [[NSAutoreleasePool alloc] init];
            loopcount = 0;
        }
    }
    [pool release];
}
//--------------------------------------------------------------

- (int)cardAddress
{
    return cardAddress;
}
//--------------------------------------------------------------

- (void)setCardAddress:(int)newCardAddress
{
	cardAddress = newCardAddress;
}
//--------------------------------------------------------------

- (BOOL)online
{
    return online;
}
//--------------------------------------------------------------

- (void)setOnline:(BOOL)newOnline
{
	online = newOnline;
}
//--------------------------------------------------------------

- (UInt8)DA1
{
    return DA1;
}

//--------------------------------------------------------------
- (void)setDA1:(UInt8)newDA1
{
	DA1 = newDA1;
}

//--------------------------------------------------------------
- (UInt8)DA2
{
    return DA2;
}

//--------------------------------------------------------------
- (void)setDA2:(UInt8)newDA2
{
	DA2 = newDA2;
}

//--------------------------------------------------------------
- (UInt8)AD1
{
    return AD1;
}

//--------------------------------------------------------------
- (void)setAD1:(UInt8)newAD1
{
	AD1 = newAD1;
}

//--------------------------------------------------------------
- (UInt8)AD2
{
    return AD2;
}

//--------------------------------------------------------------
- (void)setAD2:(UInt8)newAD2
{
	AD2 = newAD2;
}

//--------------------------------------------------------------
- (UInt8)inputValue
{
    return inputValue;
}

//--------------------------------------------------------------
- (void)setInputValue:(UInt8)newInputValue
{
	inputValue = newInputValue;
}

//--------------------------------------------------------------
- (UInt8)outputValue
{
    return outputValue;
}

//--------------------------------------------------------------
- (void)setOutputValue:(UInt8)newOutputValue
{
	outputValue = newOutputValue;
}

//--------------------------------------------------------------
- (UInt16)counter1
{
    return counter1;
}

//--------------------------------------------------------------
- (void)setCounter1:(UInt16)newCounter1
{
	counter1 = newCounter1;
}

//--------------------------------------------------------------
- (UInt16)counter2
{
    return counter2;
}

//--------------------------------------------------------------
- (void)setCounter2:(UInt16)newCounter2
{
	counter2 = newCounter2;
}

//--------------------------------------------------------------
- (BOOL)outputTest
{
    return outputTest;
}

//--------------------------------------------------------------
- (void)setOutputTest:(BOOL)newOutputTest
{
	outputTest = newOutputTest;
}
//--------------------------------------------------------------
// - K8055 commands that are sent in element 10
//		0x00 Init interface
//		0x01 Set debounce time counter 1 debounce time in element 16
//		0x02 Set debounce time counter 2 debounce time in element 17
//		0x03 reset counter 1
//		0x04 reset counter 2
//		0x05 set outputs
//--------------------------------------------------------------
- (void)resetCounter:(int)counter
{
	kern_return_t		kr;
	IOHIDEventStruct valueEvent;
	
	kr = (*k8055Interface)->getElementValue(k8055Interface, 10, &valueEvent);	
	valueEvent.value = counter+2;
	kr = (*k8055Interface)->setElementValue(k8055Interface, 10, &valueEvent,0,0,nil,nil);	

	if ((kr != kIOReturnSuccess))
	{
		printf("Unable to reset counter (%08x)\n", kr);
	}
}

//--------------------------------------------------------------
int ms2DBCode(int ms)
{
	if (ms < 2)
		return 1;
	else if (ms < 10)
		return 3;
	else if (ms < 1000)
		return 8;
	else if (ms < 5000)
		return 88;
	else
		return 255;
}
//--------------------------------------------------------------
- (void)setDebounceForCounter:(int)counter debounce:(int)ms
// set the debounce time for selected counter to dedounce milliseconds
{
	kern_return_t		kr;
	IOHIDEventStruct valueEvent;
	
	kr = (*k8055Interface)->getElementValue(k8055Interface, 10, &valueEvent);	
	valueEvent.value = counter;
	kr = (*k8055Interface)->setElementValue(k8055Interface, 10, &valueEvent,0,0,nil,nil);	
	kr = (*k8055Interface)->getElementValue(k8055Interface, 11, &valueEvent);	
	valueEvent.value = 0;
	kr = (*k8055Interface)->setElementValue(k8055Interface, 11, &valueEvent,0,0,nil,nil);	
	kr = (*k8055Interface)->getElementValue(k8055Interface, 12, &valueEvent);	
	valueEvent.value = 0;
	kr = (*k8055Interface)->setElementValue(k8055Interface, 12, &valueEvent,0,0,nil,nil);	
	kr = (*k8055Interface)->getElementValue(k8055Interface, 13, &valueEvent);	
	valueEvent.value = 0;
	kr = (*k8055Interface)->setElementValue(k8055Interface, 13, &valueEvent,0,0,nil,nil);	
	kr = (*k8055Interface)->getElementValue(k8055Interface, 14, &valueEvent);	
	valueEvent.value = 0;
	kr = (*k8055Interface)->setElementValue(k8055Interface, 14, &valueEvent,0,0,nil,nil);	
	kr = (*k8055Interface)->getElementValue(k8055Interface, 15, &valueEvent);	
	valueEvent.value = 0;
	kr = (*k8055Interface)->setElementValue(k8055Interface, 15, &valueEvent,0,0,nil,nil);	
	kr = (*k8055Interface)->getElementValue(k8055Interface, 15+counter, &valueEvent);	
	valueEvent.value = ms2DBCode(ms);
	kr = (*k8055Interface)->setElementValue(k8055Interface, 15+counter, &valueEvent,0,0,nil,nil);	

	if ((kr != kIOReturnSuccess))
	{
		printf("Unable to set debounce counter (%08x)\n", kr);
	}
}





@end
